@@ -91,7 +91,7 @@ extern int ka_Icnt;
 
 namespace XrdGlobal
 {
-XrdBuffXL xlBuff;
+extern XrdBuffXL xlBuff;
 }
 
 namespace
@@ -844,7 +844,7 @@ void XrdConfig::setCFG()
   
 int XrdConfig::setFDL()
 {
-   static const int maxFD = 1048576;
+   static const int maxFD = 65535; // was 1048576 see XrdNetAddrInfo::sockNum
    struct rlimit rlim;
    char buff[100];
 
