@@ -81,34 +81,35 @@ static XrdSysError  eDest(&Logger, "");
 
 XrdSysError  *XrdCpConfig::Log = &XrdCpConfiguration::eDest;
   
-const char   *XrdCpConfig::opLetters = ":C:d:D:fFhHI:NpPrRsS:t:T:vVX:y:Z";
+const char   *XrdCpConfig::opLetters = ":C:d:D:fFhHI:NpPrRsS:t:T:vVX:y:z:Z";
 
 struct option XrdCpConfig::opVec[] =         // For getopt_long()
      {
-      {OPT_TYPE "cksum",     1, 0, XrdCpConfig::OpCksum},
-      {OPT_TYPE "debug",     1, 0, XrdCpConfig::OpDebug},
-      {OPT_TYPE "dynamic-src",0,0, XrdCpConfig::OpDynaSrc},
-      {OPT_TYPE "coerce",    0, 0, XrdCpConfig::OpCoerce},
-      {OPT_TYPE "force",     0, 0, XrdCpConfig::OpForce},
-      {OPT_TYPE "help",      0, 0, XrdCpConfig::OpHelp},
-      {OPT_TYPE "infiles",   1, 0, XrdCpConfig::OpIfile},
-      {OPT_TYPE "license",   0, 0, XrdCpConfig::OpLicense},
-      {OPT_TYPE "nopbar",    0, 0, XrdCpConfig::OpNoPbar},
-      {OPT_TYPE "path",      0, 0, XrdCpConfig::OpPath},
-      {OPT_TYPE "posc",      0, 0, XrdCpConfig::OpPosc},
-      {OPT_TYPE "proxy",     1, 0, XrdCpConfig::OpProxy},
-      {OPT_TYPE "recursive", 0, 0, XrdCpConfig::OpRecurse},
-      {OPT_TYPE "retry",     1, 0, XrdCpConfig::OpRetry},
-      {OPT_TYPE "server",    0, 0, XrdCpConfig::OpServer},
-      {OPT_TYPE "silent",    0, 0, XrdCpConfig::OpSilent},
-      {OPT_TYPE "sources",   1, 0, XrdCpConfig::OpSources},
-      {OPT_TYPE "streams",   1, 0, XrdCpConfig::OpStreams},
-      {OPT_TYPE "tpc",       1, 0, XrdCpConfig::OpTpc},
-      {OPT_TYPE "verbose",   0, 0, XrdCpConfig::OpVerbose},
-      {OPT_TYPE "version",   0, 0, XrdCpConfig::OpVersion},
-      {OPT_TYPE "xrate",     1, 0, XrdCpConfig::OpXrate},
-      {OPT_TYPE "parallel",  1, 0, XrdCpConfig::OpParallel},
-      {0,                    0, 0, 0}
+      {OPT_TYPE "cksum",       1, 0, XrdCpConfig::OpCksum},
+      {OPT_TYPE "debug",       1, 0, XrdCpConfig::OpDebug},
+      {OPT_TYPE "dynamic-src", 0, 0, XrdCpConfig::OpDynaSrc},
+      {OPT_TYPE "coerce",      0, 0, XrdCpConfig::OpCoerce},
+      {OPT_TYPE "force",       0, 0, XrdCpConfig::OpForce},
+      {OPT_TYPE "help",        0, 0, XrdCpConfig::OpHelp},
+      {OPT_TYPE "infiles",     1, 0, XrdCpConfig::OpIfile},
+      {OPT_TYPE "license",     0, 0, XrdCpConfig::OpLicense},
+      {OPT_TYPE "nopbar",      0, 0, XrdCpConfig::OpNoPbar},
+      {OPT_TYPE "path",        0, 0, XrdCpConfig::OpPath},
+      {OPT_TYPE "posc",        0, 0, XrdCpConfig::OpPosc},
+      {OPT_TYPE "proxy",       1, 0, XrdCpConfig::OpProxy},
+      {OPT_TYPE "recursive",   0, 0, XrdCpConfig::OpRecurse},
+      {OPT_TYPE "retry",       1, 0, XrdCpConfig::OpRetry},
+      {OPT_TYPE "server",      0, 0, XrdCpConfig::OpServer},
+      {OPT_TYPE "silent",      0, 0, XrdCpConfig::OpSilent},
+      {OPT_TYPE "sources",     1, 0, XrdCpConfig::OpSources},
+      {OPT_TYPE "streams",     1, 0, XrdCpConfig::OpStreams},
+      {OPT_TYPE "tpc",         1, 0, XrdCpConfig::OpTpc},
+      {OPT_TYPE "verbose",     0, 0, XrdCpConfig::OpVerbose},
+      {OPT_TYPE "version",     0, 0, XrdCpConfig::OpVersion},
+      {OPT_TYPE "xrate",       1, 0, XrdCpConfig::OpXrate},
+      {OPT_TYPE "parallel",    1, 0, XrdCpConfig::OpParallel},
+      {OPT_TYPE "zip",         1, 0, XrdCpConfig::OpZip},
+      {0,                      0, 0, 0}
      };
 
 /******************************************************************************/
@@ -147,6 +148,7 @@ XrdCpConfig::XrdCpConfig(const char *pgm)
    inFile   = 0;
    parmVal  = 0;
    parmCnt  = 0;
+   zipFile  = 0;
 }
 
 /******************************************************************************/
@@ -162,6 +164,7 @@ XrdCpConfig::~XrdCpConfig()
    if (pHost)   free(pHost);
    if (parmVal) free(parmVal);
    if (CksMan)  delete CksMan;
+   if (zipFile) free(zipFile);
    if (dstFile) delete dstFile;
 
    while((pNow = pFile)) {pFile = pFile->Next; delete pNow;}
@@ -221,6 +224,10 @@ do{while(optind < Argc && Legacy(optind)) {}
                            break;
           case OpForce:    OpSpec |= DoForce;
                            break;
+          case OpZip:      OpSpec |= DoZip;
+                           if (zipFile) free(zipFile);
+                           zipFile = strdup(optarg);
+                           break;
           case OpHelp:     Usage(0);
                            break;
           case OpIfile:    if (inFile) free(inFile);
@@ -296,6 +303,14 @@ do{while(optind < Argc && Legacy(optind)) {}
    if (OpSpec & DoTpc &&  nSrcs > 1)
       UMSG("Third party copy requires a single source.");
 
+// Check for conflicts with ZIP archive
+//
+   if( OpSpec & DoZip & DoCksrc )
+     UMSG("Cannot calculate source checksum for a file in ZIP archive.");
+
+   if( ( OpSpec & DoZip & DoCksum ) && !CksData.HasValue() )
+     UMSG("Cannot calculate source checksum for a file in ZIP archive.");
+
 // Turn off verbose if we are in server mode
 //
    if (OpSpec & DoServer)
@@ -562,6 +577,7 @@ int XrdCpConfig::defCks(const char *opval)
    if (n >= XrdCksData::NameSize)
       UMSG("Invalid checksum type, '" <<opval <<"'.");
    strncpy(csName, opval, n); csName[n] = 0;
+   toLower( csName );
 
 // Get a checksum object for this checksum
 //
@@ -849,7 +865,7 @@ void XrdCpConfig::Usage(int rc)
    "         [--path] [--posc] [--proxy <host>:<port>] [--recursive]\n"
    "         [--retry <n>] [--server] [--silent] [--sources <n>] [--streams <n>]\n"
    "         [--tpc {first|only}] [--verbose] [--version] [--xrate <rate>]\n"
-   "         [--parallel <n>]";
+   "         [--parallel <n>] [--zip <file>]";
 
    static const char *Syntax2= "\n"
    "<src>:   [[x]root://<host>[:<port>]/]<path> | -";
@@ -894,6 +910,7 @@ void XrdCpConfig::Usage(int rc)
    "-X | --xrate <rate> limits the transfer to the specified rate. You can\n"
    "                    suffix the value with 'k', 'm', or 'g'\n"
    "     --parallel <n> number of copy jobs to be run simultaneously\n\n"
+   "-z | --zip <file>   treat the source as a ZIP archive containing given file\n"
    "Legacy options:     [-adler] [-DI<var> <val>] [-DS<var> <val>] [-np]\n"
    "                    [-md5] [-OD<cgi>] [-OS<cgi>] [-version] [-x]";
 
