@@ -44,6 +44,7 @@
 #include "XrdCks/XrdCksManOss.hh"
 #include "XrdOuc/XrdOucPinLoader.hh"
 #include "XrdOuc/XrdOucStream.hh"
+#include "XrdOuc/XrdOucUtils.hh"
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysPlugin.hh"
 
@@ -183,7 +184,7 @@ int XrdCksConfig::ParseLib(XrdOucStream &Config)
    n = strlen(val);
    if (n >= nameSize)
       {eDest->Emsg("Config", "ckslib digest name too long -", val); return 1;}
-   strcpy(buff, val); bP = buff+n; *bP++ = ' ';
+   strcpy(buff, val); XrdOucUtils::toLower(buff); bP = buff+n; *bP++ = ' ';
 
 // Get the path
 //
