@@ -286,148 +286,6 @@ namespace
       bool pMakeDir;
   };
 
-  //----------------------------------------------------------------------------
-  //! Local source
-  //----------------------------------------------------------------------------
-  class LocalSource: public Source
-  {
-    public:
-      //------------------------------------------------------------------------
-      //! Constructor
-      //------------------------------------------------------------------------
-      LocalSource( const XrdCl::URL *url, const std::string &ckSumType,
-                   uint32_t chunkSize ):
-        pPath( url->GetPath() ), pFD( -1 ), pSize( -1 ), pCurrentOffset( 0 ),
-        pCkSumHelper(0), pChunkSize( chunkSize )
-      {
-        if( !ckSumType.empty() )
-          pCkSumHelper = new CheckSumHelper( url->GetPath(), ckSumType );
-      }
-
-      //------------------------------------------------------------------------
-      //! Destructor
-      //------------------------------------------------------------------------
-      virtual ~LocalSource()
-      {
-        if( pFD != -1 )
-          close( pFD );
-        delete pCkSumHelper;
-      }
-
-      //------------------------------------------------------------------------
-      //! Initialize the source
-      //------------------------------------------------------------------------
-      virtual XrdCl::XRootDStatus Initialize()
-      {
-        using namespace XrdCl;
-        Log *log = DefaultEnv::GetLog();
-
-        if( pCkSumHelper )
-        {
-          XRootDStatus st = pCkSumHelper->Initialize();
-          if( !st.IsOK() )
-            return st;
-        }
-
-        //----------------------------------------------------------------------
-        // Open the file for reading and get it's size
-        //----------------------------------------------------------------------
-        log->Debug( UtilityMsg, "Opening %s for reading", pPath.c_str() );
-
-        int fd = open( pPath.c_str(), O_RDONLY );
-        if( fd == -1 )
-        {
-          log->Debug( UtilityMsg, "Unable to open %s: %s",
-                                  pPath.c_str(), strerror( errno ) );
-          return XRootDStatus( stError, errOSError, errno );
-        }
-
-        struct stat st;
-        if( fstat( fd, &st ) == -1 )
-        {
-          log->Debug( UtilityMsg, "Unable to stat %s: %s",
-                                  pPath.c_str(), strerror( errno ) );
-          close( fd );
-          return XRootDStatus( stError, errOSError, errno );
-        }
-        pFD   = fd;
-        pSize = st.st_size;
-
-        return XRootDStatus();
-      }
-
-      //------------------------------------------------------------------------
-      //! Get size
-      //------------------------------------------------------------------------
-      virtual int64_t GetSize()
-      {
-        return pSize;
-      }
-
-      //------------------------------------------------------------------------
-      //! Get a data chunk from the source
-      //------------------------------------------------------------------------
-      virtual XrdCl::XRootDStatus GetChunk( XrdCl::ChunkInfo &ci )
-      {
-        using namespace XrdCl;
-        Log *log = DefaultEnv::GetLog();
-
-        if( pFD == -1 )
-          return XRootDStatus( stError, errUninitialized );
-
-        const uint32_t toRead = pChunkSize;
-        char *buffer = new char[toRead];
-
-        int64_t bytesRead = read( pFD, buffer, toRead );
-        if( bytesRead == -1 )
-        {
-          log->Debug( UtilityMsg, "Unable to read from %s: %s",
-                                  pPath.c_str(), strerror( errno ) );
-          close( pFD );
-          pFD = -1;
-          delete [] buffer;
-          return XRootDStatus( stError, errOSError, errno );
-        }
-
-        if( bytesRead == 0 )
-        {
-          delete [] buffer;
-          return XRootDStatus( stOK, suDone );
-        }
-
-        if( pCkSumHelper )
-          pCkSumHelper->Update( buffer, bytesRead );
-
-        ci.offset = pCurrentOffset;
-        ci.length = bytesRead;
-        ci.buffer = buffer;
-        pCurrentOffset += bytesRead;
-        return XRootDStatus( stOK, suContinue );
-      }
-
-      //------------------------------------------------------------------------
-      //! Get check sum
-      //------------------------------------------------------------------------
-      virtual XrdCl::XRootDStatus GetCheckSum( std::string &checkSum,
-                                               std::string &checkSumType )
-      {
-        using namespace XrdCl;
-        if( pCkSumHelper )
-          return pCkSumHelper->GetCheckSum( checkSum, checkSumType );
-        return XRootDStatus( stError, errCheckSumError );
-      }
-
-    private:
-      LocalSource(const LocalSource &other);
-      LocalSource &operator = (const LocalSource &other);
-      std::string     pPath;
-      int             pFD;
-      int64_t         pSize;
-      uint64_t        pCurrentOffset;
-      CheckSumHelper *pCkSumHelper;
-      uint32_t        pChunkSize;
-  };
-
   //----------------------------------------------------------------------------
   //! StdIn source
   //----------------------------------------------------------------------------
@@ -701,6 +559,9 @@ namespace
           if( !checkSum.empty() ) return XrdCl::XRootDStatus();
         }
 
+        if( pUrl->IsLocalFile() )
+          return XrdCl::Utils::GetLocalCheckSum( checkSum, checkSumType, pUrl->GetPath() );
+
         std::string dataServer; pFile->GetProperty( "DataServer", dataServer );
         std::string lastUrl;    pFile->GetProperty( "LastURL",    lastUrl );
         return XrdCl::Utils::GetRemoteCheckSum( checkSum, checkSumType,
@@ -1212,209 +1073,7 @@ namespace
   };
 
   //----------------------------------------------------------------------------
-  //! Local destination
-  //----------------------------------------------------------------------------
-  class LocalDestination: public Destination
-  {
-    public:
-      //------------------------------------------------------------------------
-      //! Constructor
-      //------------------------------------------------------------------------
-      LocalDestination( const XrdCl::URL *url ):
-        pPath( url->GetPath() ), pFD( -1 )
-      {
-      }
-
-      //------------------------------------------------------------------------
-      //! Destructor
-      //------------------------------------------------------------------------
-      virtual ~LocalDestination()
-      {
-        if( pFD != -1 )
-          Finalize();
-      }
-
-      //------------------------------------------------------------------------
-      //! Initialize the destination
-      //------------------------------------------------------------------------
-      virtual XrdCl::XRootDStatus Initialize()
-      {
-        using namespace XrdCl;
-        Log *log = DefaultEnv::GetLog();
-
-        //----------------------------------------------------------------------
-        // Make the directory path if necessary
-        //----------------------------------------------------------------------
-        if( pMakeDir )
-        {
-          std::string dirpath = pPath.substr(0, pPath.find_last_of("/"));
-          XRootDStatus st = MkPath( dirpath );
-          if( !st.IsOK() )
-            return st;
-        }
-
-        //----------------------------------------------------------------------
-        // Open the file for reading and get it's size
-        //----------------------------------------------------------------------
-        log->Debug( UtilityMsg, "Opening %s for writing", pPath.c_str() );
-
-        int flags = O_WRONLY|O_CREAT|O_TRUNC;
-        if( !pForce )
-          flags |= O_EXCL;
-
-        int fd = open( pPath.c_str(), flags, 0644 );
-        if( fd == -1 )
-        {
-          log->Debug( UtilityMsg, "Unable to open %s: %s",
-                                  pPath.c_str(), strerror( errno ) );
-          return XRootDStatus( stError, errOSError, errno );
-        }
-
-        pFD   = fd;
-        return XRootDStatus();
-      }
-
-      //------------------------------------------------------------------------
-      //! Finalize the destination
-      //------------------------------------------------------------------------
-      virtual XrdCl::XRootDStatus Finalize()
-      {
-        using namespace XrdCl;
-        if( pFD != -1 )
-        {
-          int fd = pFD; pFD = -1;
-          if( close( fd ) != 0 )
-            return XRootDStatus( stError, errOSError, errno );
-        }
-        return XRootDStatus();
-      }
-
-      //------------------------------------------------------------------------
-      //! Put a data chunk at a destination
-      //!
-      //! @param  buffer buffer containing the chunk data
-      //! @param  ci     chunk information
-      //! @return status of the operation
-      //------------------------------------------------------------------------
-      virtual XrdCl::XRootDStatus PutChunk( XrdCl::ChunkInfo &ci )
-      {
-        using namespace XrdCl;
-        Log *log = DefaultEnv::GetLog();
-
-        if( pFD == -1 )
-          return XRootDStatus( stError, errUninitialized );
-
-        int64_t   wr     = 0;
-        uint64_t  offset = ci.offset;
-        uint32_t  length = ci.length;
-        char     *cursor = (char*)ci.buffer;
-        do
-        {
-          wr = pwrite( pFD, cursor, length, offset );
-          if( wr == -1 )
-          {
-            log->Debug( UtilityMsg, "Unable to write to %s: %s", pPath.c_str(),
-                        strerror( errno ) );
-            close( pFD );
-            pFD = -1;
-            if( pPosc )
-              unlink( pPath.c_str() );
-            delete [] (char*)ci.buffer; ci.buffer = 0;
-            return XRootDStatus( stError, errOSError, errno );
-          }
-          offset += wr;
-          cursor += wr;
-          length -= wr;
-        }
-        while( length );
-
-        delete [] (char*)ci.buffer; ci.buffer = 0;
-        return XRootDStatus();
-      }
-
-      //------------------------------------------------------------------------
-      //! Flush chunks that might have been queues
-      //------------------------------------------------------------------------
-      virtual XrdCl::XRootDStatus Flush()
-      {
-        return XrdCl::XRootDStatus();
-      }
-
-      //------------------------------------------------------------------------
-      //! Get check sum
-      //------------------------------------------------------------------------
-      virtual XrdCl::XRootDStatus GetCheckSum( std::string &checkSum,
-                                               std::string &checkSumType )
-      {
-        return XrdCl::Utils::GetLocalCheckSum( checkSum, checkSumType, pPath );
-      }
-
-      //------------------------------------------------------------------------
-      //! Create a directory path
-      //------------------------------------------------------------------------
-      virtual XrdCl::XRootDStatus MkPath( std::string &path )
-      {
-        using namespace XrdCl;
-        Log   *log = DefaultEnv::GetLog();
-        struct stat st;
-        std::vector<std::string> pathElements;
-        std::vector<std::string>::iterator it;
-        std::string fullPath;
-
-        if( path.empty() )
-          return XRootDStatus();
-
-        log->Dump( UtilityMsg, "Attempting to create %s", path.c_str() );
-
-        if( path[0] == '/' )
-          fullPath = "/";
-
-        XrdCl::Utils::splitString( pathElements, path, "/" );
-
-        for( it = pathElements.begin(); it != pathElements.end(); ++it )
-        {
-          fullPath += *it;
-          fullPath += "/";
-          if( stat( fullPath.c_str(), &st ) != 0 )
-          {
-            if( errno == ENOENT )
-            {
-              if( mkdir( fullPath.c_str(), 0755 ) != 0 )
-              {
-                log->Error( UtilityMsg, "Cannot create directory %s: %s",
-                            fullPath.c_str(), strerror( errno ) );
-                return XRootDStatus( stError, errOSError, errno );
-              }
-            }
-            else
-            {
-              log->Error( UtilityMsg, "Unable to stat %s: %s",
-                          fullPath.c_str(), strerror( errno ) );
-              return XRootDStatus( stError, errOSError, errno );
-            }
-          }
-          else if( !S_ISDIR( st.st_mode ) )
-          {
-            log->Error( UtilityMsg, "Path %s not a directory: %s",
-                        fullPath.c_str(), strerror( ENOTDIR ) );
-            return XRootDStatus( stError, errOSError, ENOTDIR );
-          }
-          else
-            log->Dump( UtilityMsg, "Path %s already exists", fullPath.c_str() );
-        }
-        return XRootDStatus();
-      }
-
-    private:
-      LocalDestination(const LocalDestination &other);
-      LocalDestination &operator = (const LocalDestination &other);
-
-      std::string pPath;
-      int         pFD;
-  };
-
-  //----------------------------------------------------------------------------
-  //! Local destination
+  //! SrdOut destination
   //----------------------------------------------------------------------------
   class StdOutDestination: public Destination
   {
@@ -1679,6 +1338,9 @@ namespace
       virtual XrdCl::XRootDStatus GetCheckSum( std::string &checkSum,
                                                std::string &checkSumType )
       {
+        if( pUrl->IsLocalFile() )
+          return XrdCl::Utils::GetLocalCheckSum( checkSum, checkSumType, pUrl->GetPath() );
+
         std::string dataServer; pFile->GetProperty( "DataServer", dataServer );
         return XrdCl::Utils::GetRemoteCheckSum( checkSum, checkSumType,
                                                 dataServer, pUrl->GetPath() );
@@ -1778,8 +1440,6 @@ namespace XrdCl
       src.reset( new XRootDSourceXCp( &GetSource(), chunkSize, parallelChunks, nbXcpSources, blockSize ) );
     else if( zip ) // TODO make zip work for xcp
       src.reset( new XRootDSourceZip( zipSource, &GetSource(), chunkSize, parallelChunks ) );
-    else if( GetSource().GetProtocol() == "file" )
-      src.reset( new LocalSource( &GetSource(), checkSumType, chunkSize ) );
     else if( GetSource().GetProtocol() == "stdio" )
       src.reset( new StdInSource( checkSumType, chunkSize ) );
     else
@@ -1797,9 +1457,7 @@ namespace XrdCl
     XRDCL_SMART_PTR_T<Destination> dest;
     URL newDestUrl( GetTarget() );
 
-    if( GetTarget().GetProtocol() == "file" )
-      dest.reset( new LocalDestination( &GetTarget() ) );
-    else if( GetTarget().GetProtocol() == "stdio" )
+    if( GetTarget().GetProtocol() == "stdio" )
       dest.reset( new StdOutDestination( checkSumType ) );
     //--------------------------------------------------------------------------
     // For xrootd destination build the oss.asize hint
