@@ -240,7 +240,6 @@ namespace XrdCl
       return pLastFatalError;
 
     gettimeofday( &pConnectionStarted, 0 );
-    pConnectionInitTime = now;
     ++pConnectionCount;
 
     //--------------------------------------------------------------------------
@@ -265,12 +264,23 @@ namespace XrdCl
     // It's more efficient to remove addresses from the back of a vector
     // so we reverse the it.
     //--------------------------------------------------------------------------
-    std::reverse( pAddresses.begin(), pAddresses.end() );
-    pSubStreams[0]->socket->SetAddress( pAddresses.back() );
-    pAddresses.pop_back();
-    st = pSubStreams[0]->socket->Connect( pConnectionWindow );
-    if( st.IsOK() )
-      pSubStreams[0]->status = Socket::Connecting;
+    int preferIPv4 = DefaultPreferIPv4;
+    DefaultEnv::GetEnv()->GetInt( "PreferIPv4", preferIPv4 );
+    if( !preferIPv4 )
+      std::reverse( pAddresses.begin(), pAddresses.end() );
+
+    while( !pAddresses.empty() )
+    {
+      pSubStreams[0]->socket->SetAddress( pAddresses.back() );
+      pAddresses.pop_back();
+      pConnectionInitTime = ::time( 0 );
+      st = pSubStreams[0]->socket->Connect( pConnectionWindow );
+      if( st.IsOK() )
+      {
+        pSubStreams[0]->status = Socket::Connecting;
+        break;
+      }
+    }
     return st;
   }
 
@@ -325,27 +335,6 @@ namespace XrdCl
     return st;
   }
 
-
-  //------------------------------------------------------------------------
-  // Queue a virtual response
-  //------------------------------------------------------------------------
-  Status Stream::ReceiveVirtual( Message *msg )
-  {
-    //--------------------------------------------------------------------------
-    // Check the session ID and bounce if needed
-    //--------------------------------------------------------------------------
-    if( msg->GetSessionId() &&
-        (pSubStreams[0]->status != Socket::Connected ||
-        pSessionId != msg->GetSessionId()) )
-      return Status( stError, errInvalidSession );
-
-    Log *log = DefaultEnv::GetLog();
-    log->Dump( PostMasterMsg, "[%s] Queuing virtual response: 0x%x.", pStreamName.c_str(), msg );
-    pJobManager->QueueJob( pQueueIncMsgJob, msg );
-
-    return Status();
-  }
-
   //----------------------------------------------------------------------------
   // Force connection
   //----------------------------------------------------------------------------
@@ -446,7 +435,7 @@ namespace XrdCl
     params.followRedirects = false;
     params.stateful        = true;
     MessageUtils::ProcessSendParams( params );
-    return MessageUtils::SendMessage( *pUrl, msg, handler, params );
+    return MessageUtils::SendMessage( *pUrl, msg, handler, params, 0 );
   }
 
   //----------------------------------------------------------------------------
@@ -689,48 +678,48 @@ namespace XrdCl
     // Check if we still have time to try and do something in the current window
     //--------------------------------------------------------------------------
     time_t elapsed = now-pConnectionInitTime;
-    if( elapsed < pConnectionWindow )
+    log->Error( PostMasterMsg, "[%s] elapsed = %d, pConnectionWindow = %d "
+               "seconds.", pStreamName.c_str(), elapsed, pConnectionWindow );
+
+    //------------------------------------------------------------------------
+    // If we have some IP addresses left we try them
+    //------------------------------------------------------------------------
+    if( !pAddresses.empty() )
     {
-      //------------------------------------------------------------------------
-      // If we have some IP addresses left we try them
-      //------------------------------------------------------------------------
-      if( !pAddresses.empty() )
+      Status st;
+      do
       {
         pSubStreams[0]->socket->SetAddress( pAddresses.back() );
         pAddresses.pop_back();
-
-        Status st = pSubStreams[0]->socket->Connect( pConnectionWindow-elapsed );
-        if( !st.IsOK() )
-          OnFatalError( subStream, st, scopedLock );
-        return;
+        pConnectionInitTime = ::time( 0 );
+        st = pSubStreams[0]->socket->Connect( pConnectionWindow );
       }
+      while( !pAddresses.empty() && !st.IsOK() );
 
-      //------------------------------------------------------------------------
-      // If we still can retry with the same host name, we sleep until the end
-      // of the connection window and try
-      //------------------------------------------------------------------------
-      else if( pConnectionCount < pConnectionRetry && !status.IsFatal() )
-      {
-        log->Info( PostMasterMsg, "[%s] Attempting reconnection in %d "
-                   "seconds.", pStreamName.c_str(), pConnectionWindow-elapsed );
+      if( !st.IsOK() )
+        OnFatalError( subStream, st, scopedLock );
 
-        Task *task = new ::StreamConnectorTask( this );
-        pTaskManager->RegisterTask( task, pConnectionInitTime+pConnectionWindow );
-        return;
-      }
-
-      //------------------------------------------------------------------------
-      // Nothing can be done, we declare a failure
-      //------------------------------------------------------------------------
-      OnFatalError( subStream, status, scopedLock );
       return;
     }
+    //------------------------------------------------------------------------
+    // If we still can retry with the same host name, we sleep until the end
+    // of the connection window and try
+    //------------------------------------------------------------------------
+    else if( elapsed < pConnectionWindow && pConnectionCount < pConnectionRetry
+             && !status.IsFatal() )
+    {
+      log->Info( PostMasterMsg, "[%s] Attempting reconnection in %d "
+                 "seconds.", pStreamName.c_str(), pConnectionWindow-elapsed );
 
+      Task *task = new ::StreamConnectorTask( this );
+      pTaskManager->RegisterTask( task, pConnectionInitTime+pConnectionWindow );
+      return;
+    }
     //--------------------------------------------------------------------------
     // We are out of the connection window, the only thing we can do here
     // is re-resolving the host name and retrying if we still can
     //--------------------------------------------------------------------------
-    if( pConnectionCount < pConnectionRetry && !status.IsFatal() )
+    else if( pConnectionCount < pConnectionRetry && !status.IsFatal() )
     {
       pAddresses.clear();
       pSubStreams[0]->status = Socket::Disconnected;
