@@ -60,7 +60,8 @@ struct XrdCnsSsiFRec
 {
 char Info[XrdCnsLogRec::FixDLen];
 
-void Updt(const char *nInfo) {strncpy(Info, nInfo, sizeof(Info));}
+void Updt(const char *nInfo) {
+  if(nInfo != 0) strncpy(Info, nInfo, sizeof(Info));}
 
      XrdCnsSsiFRec(const char *Data) {if (!Data) Data = XrdCnsLogRec::iArg;
                                       strncpy(Info, Data, sizeof(Info));
@@ -544,8 +545,8 @@ void XrdCnsSsi::ApplyLogRec(char *lP)
                                            theDir->Files->Del(fnP);
                                                                 break;
            case XrdCnsLogRec::lrRmdir:  hInv->Del(lfn);         break;
-           case XrdCnsLogRec::lrMv:     if (AddDel(lfn, lP))    break;
-           default: if (fnP)  *(fnP -1) = '/';
+           default: if (Type == XrdCnsLogRec::lrMv && AddDel(lfn, lP)) break;
+                    if (fnP)  *(fnP -1) = '/';
                     Say.V("Invalid log record ", lP);
                     nErrs++;
           }
