@@ -62,6 +62,7 @@ XrdSysMutex *XrdCryptosslFactory::CryptoMutexPool[SSLFACTORY_MAX_CRYPTO_MUTEX];
 /******************************************************************************/
 /*             T h r e a d - S a f e n e s s   F u n c t i o n s              */
 /******************************************************************************/
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
 #ifdef __solaris__
 extern "C" {
 #endif
@@ -69,7 +70,7 @@ static unsigned long sslfactory_id_callback(void) {
   return (unsigned long)XrdSysThread::ID();
 }
 
-void sslfactory_lock(int mode, int n, const char *file, int line)
+static void sslfactory_lock(int mode, int n, const char *file, int line)
 {
   if (mode & CRYPTO_LOCK) {
     if (XrdCryptosslFactory::CryptoMutexPool[n]) {
@@ -84,6 +85,7 @@ void sslfactory_lock(int mode, int n, const char *file, int line)
 #ifdef __solaris__
 }
 #endif
+#endif
 
 
 //______________________________________________________________________________
@@ -497,6 +499,14 @@ XrdCryptoX509SignProxyReq_t XrdCryptosslFactory::X509SignProxyReq()
    return &XrdCryptosslX509SignProxyReq;
 }
 
+//______________________________________________________________________________
+XrdCryptoX509CheckProxy3_t XrdCryptosslFactory::X509CheckProxy3()
+{
+   // Check consistency of a GSI 3 compliant proxy
+
+   return &XrdCryptosslX509CheckProxy3;
+}
+
 //______________________________________________________________________________
 XrdCryptoX509GetVOMSAttr_t XrdCryptosslFactory::X509GetVOMSAttr()
 {
