@@ -776,9 +776,11 @@ int XrdHttpReq::ProcessHTTPReq() {
   
   
   // Verify if we have an external handler for this request
-  if (prot->exthandler && prot->exthandler->MatchesPath(this->resource.c_str())) {
+
+  XrdHttpExtHandler *exthandler = prot->FindMatchingExtHandler(*this);
+  if (exthandler) {
     XrdHttpExtReq xreq(this, prot);
-    int r = prot->exthandler->ProcessReq(xreq);
+    int r = exthandler->ProcessReq(xreq);
     reset();
     if (!r) return 1; // All went fine, response sent
     if (r < 0) return -1; // There was a hard error... close the connection
@@ -1088,7 +1090,7 @@ int XrdHttpReq::ProcessHTTPReq() {
         l = resourceplusopaque.length() + 1;
         xrdreq.open.dlen = htonl(l);
         xrdreq.open.mode = htons(kXR_ur | kXR_uw | kXR_gw | kXR_gr | kXR_or);
-        xrdreq.open.options = htons(kXR_mkpath | kXR_open_updt | kXR_new);
+        xrdreq.open.options = htons(kXR_mkpath | kXR_open_wrto );
 
         if (!prot->Bridge->Run((char *) &xrdreq, (char *) resourceplusopaque.c_str(), l)) {
           prot->SendSimpleResp(404, NULL, NULL, (char *) "Could not run request.", 0);
