@@ -482,6 +482,14 @@ bool XrdSsiTaskReal::SendRequest(const char *node)
    rrInfo.Size(reqBlen);
    tStat = isWrite;
 
+// If we are writing a zero length message, we must fake a request as zero
+// zero length messages are normally deep-sixed.
+//
+   if (!reqBlen)
+      {reqBuff = &zedData;
+       reqBlen = 1;
+      }
+
 // Issue the write
 //
    Status = sessP->epFile.Write(rrInfo.Info(), (uint32_t)reqBlen, reqBuff,
