@@ -284,7 +284,7 @@ void *XrdSysPlugin::getPlugin(const char *pname, int optional, bool global)
 //
    if (!myHandle)
       {if ((myHandle = dlopen(libPath, flags))) libHandle = myHandle;
-          else {libMsg(dlerror(), " loading "); return 0;}
+          else {if (optional < 2) libMsg(dlerror(), " loading "); return 0;}
       }
 
 // Get the symbol. In the environment we have defined, null values are not
