@@ -73,6 +73,7 @@
 #include "XrdXrootd/XrdXrootdProtocol.hh"
 #include "XrdXrootd/XrdXrootdStats.hh"
 #include "XrdXrootd/XrdXrootdTrace.hh"
+#include "XrdXrootd/XrdXrootdTransit.hh"
 #include "XrdXrootd/XrdXrootdXPath.hh"
 
 #include "Xrd/XrdBuffer.hh"
@@ -145,7 +146,6 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
 
    XrdOucEnv myEnv;
    XrdXrootdXPath *xp;
-   XrdSecGetProt_t secGetProt = 0;
    char *adminp, *rdf, *bP, *tmp, c, buff[1024];
    int i, n, deper = 0;
 
@@ -189,9 +189,13 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
      { switch(c)
        {
        case 'r': deper = 1;
+                 XrdOucEnv::Export("XRDREDIRECT", "R");
+                 break;
        case 'm': XrdOucEnv::Export("XRDREDIRECT", "R");
                  break;
        case 't': deper = 1;
+                 XrdOucEnv::Export("XRDRETARGET", "1");
+                 break;
        case 's': XrdOucEnv::Export("XRDRETARGET", "1");
                  break;
        case 'y': XrdOucEnv::Export("XRDREDPROXY", "1");
@@ -251,15 +255,7 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
 
 // Initialize the security system if this is wanted
 //
-   if (!SecLib) eDest.Say("Config warning: 'xrootd.seclib' not specified;"
-                          " strong authentication disabled!");
-      else {TRACE(DEBUG, "Loading security library " <<SecLib);
-            if (!(CIA = XrdSecLoadSecService(&eDest, pi->ConfigFN,
-                        (strcmp(SecLib,"default") ? SecLib : 0), &secGetProt)))
-               {eDest.Emsg("Config", "Unable to load security system.");
-                return 0;
-               }
-           }
+   if (!ConfigSecurity(myEnv, pi->ConfigFN)) return 0;
 
 // Set up the network for self-identification and display it
 //
@@ -270,7 +266,6 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
 //
    myEnv.PutPtr("XrdInet*", (void *)(pi->NetTCP));
    myEnv.PutPtr("XrdNetIF*", (void *)(&(pi->NetTCP->netIF)));
-   myEnv.PutPtr("XrdSecGetProtocol*", (void *)secGetProt);
    myEnv.PutPtr("XrdScheduler*", Sched);
 
 // Copy over the xrd environment which contains plugin argv's
@@ -342,10 +337,12 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
    XrdXrootdFile::Init(Locker, as_nosf == 0);
    if (as_nosf) eDest.Say("Config warning: sendfile I/O has been disabled!");
 
-// Schedule protocol object cleanup
+// Schedule protocol object cleanup (also advise the transit protocol)
 //
    ProtStack.Set(pi->Sched, XrdXrootdTrace, TRACE_MEM);
-   ProtStack.Set((pi->ConnMax/3 ? pi->ConnMax/3 : 30), 60*60);
+   n = (pi->ConnMax/3 ? pi->ConnMax/3 : 30);
+   ProtStack.Set(n, 60*60);
+   XrdXrootdTransit::Init(pi->Sched, n, 60*60);
 
 // Initialize the request ID generation object
 //
@@ -367,7 +364,6 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
        if (!strcmp(rdf, "M"))  myRole |=kXR_attrMeta;
       } 
    if (getenv("XRDREDPROXY"))  myRole |=kXR_attrProxy;
-   myRole = htonl(myRole); myRolf = htonl(myRolf);
 
 // Check if we are redirecting anything
 //
@@ -439,6 +435,17 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
 //
    PidFile();
 
+// Finally, check if we really need to be in bypass mode if it is set
+//
+   if (OD_Bypass)
+      {const char *penv = getenv("XRDXROOTD_PROXY");
+       if (!penv || *penv != '=')
+          {OD_Bypass = false;
+           eDest.Say("Config warning: 'fsoverload bypass' ignored; "
+                                     "not a forwarding proxy.");
+          }
+      }
+
 // Return success
 //
    free(adminp);
@@ -478,6 +485,7 @@ int XrdXrootdProtocol::Config(const char *ConfigFN)
              else if TS_Xeq("diglib",        xdig);
              else if TS_Xeq("export",        xexp);
              else if TS_Xeq("fslib",         xfsl);
+             else if TS_Xeq("fsoverload",    xfso);
              else if TS_Xeq("log",           xlog);
              else if TS_Xeq("monitor",       xmon);
              else if TS_Xeq("pidpath",       xpidf);
@@ -485,6 +493,7 @@ int XrdXrootdProtocol::Config(const char *ConfigFN)
              else if TS_Xeq("redirect",      xred);
              else if TS_Xeq("seclib",        xsecl);
              else if TS_Xeq("trace",         xtrace);
+             else if TS_Xeq("limit",         xlimit);
              else {eDest.Say("Config warning: ignoring unknown directive '",var,"'.");
                    Config.Echo();
                    continue;
@@ -525,6 +534,44 @@ void XrdXrootdProtocol::PidFile()
     if (xop) eDest.Emsg("Config", errno, xop, pidFN);
 }
 
+/******************************************************************************/
+/*                        C o n f i g S e c u r i t y                         */
+/******************************************************************************/
+
+int XrdXrootdProtocol::ConfigSecurity(XrdOucEnv &xEnv, const char *cfn)
+{
+   XrdSecGetProt_t secGetProt = 0;
+
+// Check if we need to loadanything
+//
+   if (!SecLib)
+      {eDest.Say("Config warning: 'xrootd.seclib' not specified;"
+                 " strong authentication disabled!");
+       xEnv.PutPtr("XrdSecGetProtocol*", (void *)0);
+       xEnv.PutPtr("XrdSecProtector*"  , (void *)0);
+       return 1;
+      }
+
+// Blad some debugging info
+//
+   TRACE(DEBUG, "Loading security library " <<SecLib);
+
+// Load the security server
+//
+   if (!(CIA = XrdSecLoadSecService(&eDest, cfn,
+               (strcmp(SecLib,"default") ? SecLib : 0),
+               &secGetProt, &DHS)))
+      {eDest.Emsg("Config", "Unable to load security system.");
+       return 0;
+      }
+
+// Set environmental pointers
+//
+   xEnv.PutPtr("XrdSecGetProtocol*", (void *)secGetProt);
+   xEnv.PutPtr("XrdSecProtector*"  , (void *)DHS);
+   return 1;
+}
+  
 /******************************************************************************/
 /*                                x a s y n c                                 */
 /******************************************************************************/
@@ -678,7 +725,8 @@ int XrdXrootdProtocol::xcksum(XrdOucStream &Config)
    while ((palg = Config.GetWord()) && *palg != '/')
          {if (!strcmp(palg,"chkcgi")) {JobCKCGI = 1; continue;}
           if (strcmp(palg, "max"))
-             {XrdOucTList *xalg = new XrdOucTList(palg, anum); anum[0]++;
+             {XrdOucUtils::toLower(palg);
+              XrdOucTList *xalg = new XrdOucTList(palg, anum); anum[0]++;
               if (algLast) algLast->next = xalg;
                  else      algFirst      = xalg;
               algLast = xalg;
@@ -929,11 +977,86 @@ int XrdXrootdProtocol::xfsL(XrdOucStream &Config, char *val, int lix)
    if (!(Slash = rindex(val, '/'))) Slash = val;
       else Slash++;
    if (!strcmp(Slash, "libXrdOfs.so"))
-      eDest.Say("Config warning: ignoring fslib; libXrdOfs.so is built-in.");
+      eDest.Say("Config warning: 'fslib libXrdOfs.so' is actually built-in.");
       else {FSLib[lix] = strdup(val); FSLvn[lix] = lvn;}
    return 0;
 }
 
+/******************************************************************************/
+/*                                  x f s o                                   */
+/******************************************************************************/
+  
+/* Function: xfso
+
+   Purpose:  To parse the directive: fsoverload [options]
+
+   options:  [[no]bypass] [redirect <host>:<port>[%<prvhost>:<port>]]
+             [stall <sec>]
+
+             bypass    If path is a forwarding path, redirect client to the
+                       location specified in the path to bypass this server.
+                       The default is nobypass.
+             redirect  Redirect the request to the specified destination.
+             stall     Stall the client <sec> seconds. The default is 33.
+*/
+
+int XrdXrootdProtocol::xfso(XrdOucStream &Config)
+{
+    static const int rHLen = 264;
+    char rHost[2][rHLen], *hP[2] = {0,0}, *val;
+    int  rPort[2], bypass = -1, stall = -1;
+
+// Process all of the options
+//
+   while((val = Config.GetWord()) && *val)
+        {     if (!strcmp(val, "bypass"))   bypass = 1;
+         else if (!strcmp(val, "nobypass")) bypass = 0;
+         else if (!strcmp(val, "redirect"))
+                 {val = Config.GetWord();
+                  if (!xred_php(val, hP, rPort)) return 1;
+                  for (int i = 0; i < 2; i++)
+                      {if (!hP[i]) rHost[i][0] = 0;
+                          else {strlcpy(rHost[i], hP[i], rHLen);
+                                hP[i] = rHost[i];
+                               }
+                      }
+                 }
+         else if (!strcmp(val, "stall"))
+                 {if (!(val = Config.GetWord()) || !(*val))
+                     {eDest.Emsg("Config", "stall value not specified");
+                      return 1;
+                     }
+                  if (XrdOuca2x::a2tm(eDest,"stall",val,&stall,0,32767))
+                     return 1;
+                 }
+         else {eDest.Emsg("config","invalid fsoverload option",val); return 1;}
+        }
+
+// Set all specified values
+//
+   if (bypass >= 0) OD_Bypass = (bypass ? true : false);
+   if (stall  >= 0) OD_Stall  = stall;
+   if (hP[0])
+      {if (Route[RD_ovld].Host[0]) free(Route[RD_ovld].Host[0]);
+       if (Route[RD_ovld].Host[1]) free(Route[RD_ovld].Host[1]);
+       Route[RD_ovld].Host[0] = strdup(hP[0]);
+       Route[RD_ovld].Port[0] = rPort[0];
+       Route[RD_ovld].RDSz[0] = strlen(hP[0]);
+       if (hP[1])
+          {Route[RD_ovld].Host[1] = strdup(hP[1]);
+           Route[RD_ovld].Port[1] = rPort[1];
+           Route[RD_ovld].RDSz[1] = strlen(hP[1]);
+          } else {
+           Route[RD_ovld].Host[1] = Route[RD_ovld].Host[0];
+           Route[RD_ovld].Port[1] = Route[RD_ovld].Port[0];
+           Route[RD_ovld].RDSz[1] = Route[RD_ovld].RDSz[0];
+          }
+       OD_Redir = true;
+      } else OD_Redir = false;
+
+   return 0;
+}
+
 /******************************************************************************/
 /*                                  x l o g                                   */
 /******************************************************************************/
@@ -1301,38 +1424,22 @@ int XrdXrootdProtocol::xred(XrdOucStream &Config)
         {"trunc",    RD_trunc}
        };
     static const int rHLen = 264;
-    char rHost[2][rHLen], *hP[2], *val, *pp;
+    char rHost[2][rHLen], *hP[2], *val;
     int i, k, neg, numopts = sizeof(rdopts)/sizeof(struct rediropts);
     int rPort[2], isQ = 0;
 
 // Get the host and port
 //
    val = Config.GetWord();
+   if (!xred_php(val, hP, rPort)) return 1;
 
-// Check if we have two hosts here
-//
-   hP[0] = val;
-   if (!(pp = index(val, '%'))) hP[1] = 0;
-      else {hP[1] = pp+1; *pp = 0;}
-
-// Verify corectness here
-//
-   if (!(*val) || (hP[1] && !hP[1]))
-      {eDest.Emsg("Config", "malformed redirect host specification"); return 1;}
-
-// Process the hosts
+// Copy out he values as the target variable will be lost
 //
    for (i = 0; i < 2; i++)
-       {if (!(val = hP[i])) break;
-        if (!val || !val[0] || val[0] == ':')
-           {eDest.Emsg("Config", "redirect host not specified"); return 1;}
-        if (!(pp = rindex(val, ':')))
-           {eDest.Emsg("Config", "redirect port not specified"); return 1;}
-        if (!(rPort[i] = atoi(pp+1)))
-           {eDest.Emsg("Config", "redirect port is invalid");    return 1;}
-        *pp = '\0';
-        strlcpy(rHost[i], val, rHLen);
-        hP[i] = rHost[i];
+       {if (!hP[i]) rHost[i][0] = 0;
+           else {strlcpy(rHost[i], hP[i], rHLen);
+                 hP[i] = rHost[i];
+                }
        }
 
 // Set all redirect target functions
@@ -1388,6 +1495,45 @@ int XrdXrootdProtocol::xred(XrdOucStream &Config)
    return 0;
 }
 
+bool XrdXrootdProtocol::xred_php(char *val, char *hP[2], int rPort[2])
+{
+   char *pp;
+
+// Make sure we have a value
+//
+   if (!val || !(*val))
+      {eDest.Emsg("config", "redirect option not specified"); return false;}
+
+// Check if we have two hosts here
+//
+   hP[0] = val;
+   if (!(pp = index(val, '%'))) hP[1] = 0;
+      else {hP[1] = pp+1; *pp = 0;}
+
+// Verify corectness here
+//
+   if (!(*val) || (hP[1] && !*hP[1]))
+      {eDest.Emsg("Config", "malformed redirect host specification");
+       return false;
+      }
+
+// Process the hosts
+//
+   for (int i = 0; i < 2; i++)
+       {if (!(val = hP[i])) break;
+        if (!val || !val[0] || val[0] == ':')
+           {eDest.Emsg("Config", "redirect host not specified"); return false;}
+        if (!(pp = rindex(val, ':')))
+           {eDest.Emsg("Config", "redirect port not specified"); return false;}
+        if (!(rPort[i] = atoi(pp+1)))
+           {eDest.Emsg("Config", "redirect port is invalid");    return false;}
+        *pp = '\0';
+       }
+
+// All done
+//
+   return true;
+}
 
 void XrdXrootdProtocol::xred_set(RD_func func, char *rHost[2], int rPort[2])
 {
@@ -1513,3 +1659,42 @@ int XrdXrootdProtocol::xtrace(XrdOucStream &Config)
     XrdXrootdTrace->What = trval;
     return 0;
 }
+
+/******************************************************************************/
+/*                                x l i m i t                                 */
+/******************************************************************************/
+
+/* Function: xlimit
+
+   Purpose:  To parse the directive: limit [prepare <count>] [noerror]
+
+             prepare <count> The maximum number of prepares that are allowed
+                             during the course of a single connection
+
+             noerror         When possible, do not issue an error when a limit
+                             is hit.
+
+   Output: 0 upon success or 1 upon failure.
+*/
+int XrdXrootdProtocol::xlimit(XrdOucStream &Config)
+{
+   int plimit = -1;
+   const char *word;
+
+// Look for various limits set
+//
+   while ( (word = Config.GetWord()) ) {
+      if (!strcmp(word, "prepare")) {
+          if (!(word = Config.GetWord()))
+          {
+             eDest.Emsg("Config", "'limit prepare' value not specified");
+             return 1;
+          }
+          if (XrdOuca2x::a2i(eDest, "limit prepare", word, &plimit, 0)) { return 1; }
+      } else if (!strcmp(word, "noerror")) {
+          LimitError = false;
+      }
+   }
+   if (plimit >= 0) {PrepareLimit = plimit;}
+   return 0;
+}
