@@ -116,6 +116,7 @@ int XrdXrootdTransit::Attn(XrdLink *lP, short *theSID, int rcode,
 int XrdXrootdTransit::AttnCont(XrdXrootdTransPend *tP,  int rcode,
                                const struct iovec *ioV, int ioN, int ioL)
 {
+   XrdLink *theLink = tP->link;
    int rc;
 
 // Refresh the request structure
@@ -143,7 +144,7 @@ int XrdXrootdTransit::AttnCont(XrdXrootdTransPend *tP,  int rcode,
           {AtomicBeg(runMutex);
            AtomicZAP(runStatus);
            AtomicEnd(runMutex);
-           tP->link->Enable();
+           theLink->Enable();
           } else {
            if (runWait >= 0)
               Sched->Schedule((XrdJob *)&waitJob, time(0)+runWait);
