@@ -46,6 +46,7 @@ class FileTest: public CppUnit::TestCase
       CPPUNIT_TEST( RedirectReturnTest );
       CPPUNIT_TEST( ReadTest );
       CPPUNIT_TEST( WriteTest );
+      CPPUNIT_TEST( WriteVTest );
       CPPUNIT_TEST( VectorReadTest );
       CPPUNIT_TEST( VirtualRedirectorTest );
       CPPUNIT_TEST( PlugInTest );
@@ -53,6 +54,7 @@ class FileTest: public CppUnit::TestCase
     void RedirectReturnTest();
     void ReadTest();
     void WriteTest();
+    void WriteVTest();
     void VectorReadTest();
     void VirtualRedirectorTest();
     void PlugInTest();
@@ -112,7 +114,7 @@ void FileTest::RedirectReturnTest()
   MessageSendParams params; params.followRedirects = false;
   MessageUtils::ProcessSendParams( params );
   OpenInfo *response = 0;
-  CPPUNIT_ASSERT_XRDST( MessageUtils::SendMessage( url, msg, handler, params ) );
+  CPPUNIT_ASSERT_XRDST( MessageUtils::SendMessage( url, msg, handler, params, 0 ) );
   XRootDStatus st1 = MessageUtils::WaitForResponse( handler, response );
   delete handler;
   CPPUNIT_ASSERT_XRDST_NOTOK( st1, errRedirect );
@@ -329,6 +331,88 @@ void FileTest::WriteTest()
   delete stat;
 }
 
+//------------------------------------------------------------------------------
+// WriteV test
+//------------------------------------------------------------------------------
+void FileTest::WriteVTest()
+{
+  using namespace XrdCl;
+
+  //----------------------------------------------------------------------------
+  // Initialize
+  //----------------------------------------------------------------------------
+  Env *testEnv = TestEnv::GetEnv();
+
+  std::string address;
+  std::string dataPath;
+
+  CPPUNIT_ASSERT( testEnv->GetString( "MainServerURL", address ) );
+  CPPUNIT_ASSERT( testEnv->GetString( "DataPath", dataPath ) );
+
+  URL url( address );
+  CPPUNIT_ASSERT( url.IsValid() );
+
+  std::string filePath = dataPath + "/testFile.dat";
+  std::string fileUrl = address + "/";
+  fileUrl += filePath;
+
+  //----------------------------------------------------------------------------
+  // Fetch some data and checksum
+  //----------------------------------------------------------------------------
+  const uint32_t MB = 1024*1024;
+  char *buffer1 = new char[4*MB];
+  char *buffer2 = new char[4*MB];
+  char *buffer3 = new char[8*MB];
+  uint32_t bytesRead1 = 0;
+  File f1, f2;
+
+  CPPUNIT_ASSERT( Utils::GetRandomBytes( buffer1, 4*MB ) == 4*MB );
+  CPPUNIT_ASSERT( Utils::GetRandomBytes( buffer2, 4*MB ) == 4*MB );
+  uint32_t crc1 = Utils::ComputeCRC32( buffer1, 4*MB );
+  crc1 = Utils::UpdateCRC32( crc1, buffer2, 4*MB );
+
+  //----------------------------------------------------------------------------
+  // Prepare IO vector
+  //----------------------------------------------------------------------------
+  int iovcnt = 2;
+  iovec iov[iovcnt];
+  iov[0].iov_base = buffer1;
+  iov[0].iov_len  = 4*MB;
+  iov[1].iov_base = buffer2;
+  iov[1].iov_len  = 4*MB;
+
+  //----------------------------------------------------------------------------
+  // Write the data
+  //----------------------------------------------------------------------------
+  CPPUNIT_ASSERT( f1.Open( fileUrl, OpenFlags::Delete | OpenFlags::Update,
+                           Access::UR | Access::UW ).IsOK() );
+  CPPUNIT_ASSERT( f1.WriteV( 0, iov, iovcnt ).IsOK() );
+  CPPUNIT_ASSERT( f1.Sync().IsOK() );
+  CPPUNIT_ASSERT( f1.Close().IsOK() );
+
+  //----------------------------------------------------------------------------
+  // Read the data and verify the checksums
+  //----------------------------------------------------------------------------
+  StatInfo *stat = 0;
+  CPPUNIT_ASSERT( f2.Open( fileUrl, OpenFlags::Read ).IsOK() );
+  CPPUNIT_ASSERT( f2.Stat( false, stat ).IsOK() );
+  CPPUNIT_ASSERT( stat );
+  CPPUNIT_ASSERT( stat->GetSize() == 8*MB );
+  CPPUNIT_ASSERT( f2.Read( 0, 8*MB, buffer3, bytesRead1 ).IsOK() );
+  CPPUNIT_ASSERT( bytesRead1 == 8*MB );
+
+  uint32_t crc2 = Utils::ComputeCRC32( buffer3, 8*MB );
+  CPPUNIT_ASSERT( f2.Close().IsOK() );
+  CPPUNIT_ASSERT( crc1 == crc2 );
+
+  FileSystem fs( url );
+  CPPUNIT_ASSERT( fs.Rm( filePath ).IsOK() );
+  delete [] buffer1;
+  delete [] buffer2;
+  delete [] buffer3;
+  delete stat;
+}
+
 //------------------------------------------------------------------------------
 // Vector read test
 //------------------------------------------------------------------------------
